from __future__ import absolute_import
from builtins import range
from builtins import object
# Copyright (c) 2015, Disney Research
# All rights reserved.
#
# Author(s): Sehoon Ha <sehoon.ha@disneyresearch.com>
# Disney Research Robotics Group


import os.path
import numpy as np
from . import pydart2_api as papi
from .skeleton import Skeleton
from .collision_result import CollisionResult


def create_world(step, skel_path=None):
    skel_path = os.path.realpath(skel_path)
    return World(step, skel_path)


class World(object):
    DART_COLLISION_DETECTOR, \
        FCL_COLLISION_DETECTOR, \
        BULLET_COLLISION_DETECTOR = list(range(3))

    def __init__(self, step, skel_path=None):
        self.skeletons = list()
        self.control_skel = None
        if skel_path is not None:
            skel_path = os.path.realpath(skel_path)
            self.id = papi.createWorldFromSkel(skel_path)
            self.set_time_step(step)
            nskels = self.num_skeletons()
            for i in range(nskels):
                self.add_skeleton_from_id(i)
        else:
            self.id = papi.createWorld(step)

        self.reset()

    def destroy(self):
        papi.destroyWorld(self.id)

    def add_skeleton(self, filename):
        skel = Skeleton(self, _filename=filename)
        self.skeletons.append(skel)
        return skel

    def add_skeleton_from_id(self, _skel_id):
        skel = Skeleton(_world=self, _id=_skel_id)
        self.skeletons.append(skel)
        return skel

    def num_skeletons(self):
        return papi.world__getNumSkeletons(self.id)

    @property
    def skel(self):
        """ returns the default control skeleton """
        return self.control_skel

    def time(self):
        return papi.world__getTime(self.id)

    @property
    def t(self):
        return self.time()

    def time_step(self):
        return papi.world__getTimeStep(self.id)

    @property
    def frame(self):
        return self._frame

    @property
    def dt(self):
        return self.time_step()

    def set_time_step(self, _time_step):
        papi.world__setTimeStep(self.id, _time_step)

    @dt.setter
    def dt(self, _dt):
        self.set_time_step(_dt)

    def num_frames(self):
        return papi.world__getSimFrames(self.id)

    @property
    def nframes(self):
        return self.num_frames()

    def reset(self):
        papi.world__reset(self.id)
        self._frame = 0
        self.collision_result = CollisionResult(self)

    def step(self):
        for skel in self.skeletons:
            if skel.controller is not None:
                skel.tau = skel.controller.compute()

        papi.world__step(self.id)
        self._frame += 1
        self.collision_result.update()

    def check_collision(self, ):
        papi.world__checkCollision(self.id)
        self.collision_result.update()

    def render(self,
               render_markers=True,
               render_contacts=True,
               render_contact_size=0.01,
               render_contact_force_scale=-0.005):
        papi.world__render(self.id)
        if render_markers:
            self.render_markers()

        if render_contacts:
            self.render_contacts(render_contact_size,
                                 render_contact_force_scale)

    def render_markers(self, ):
        for skel in self.skeletons:
            for marker in skel.markers:
                marker.render()

    def render_contacts(self,
                        render_contact_size=0.01,
                        render_contact_force_scale=-0.005):
        for contact in self.collision_result.contacts:
            contact.render(size=render_contact_size,
                           scale=render_contact_force_scale)

    def states(self):
        return np.concatenate([skel.x for skel in self.skels])

    @property
    def x(self):
        return self.states()

    def set_states(self, _x):
        lo = 0
        for skel in self.skels:
            hi = lo + 2 * skel.ndofs
            skel.x = _x[lo:hi]
            lo = hi

    @x.setter
    def x(self, _x):
        self.set_states(_x)

    def gravity(self):
        return papi.world__getGravity(self.id)

    @property
    def g(self):
        return self.gravity()

    def set_gravity(self, _g):
        papi.world__setGravity(self.id, _g)

    @g.setter
    def g(self, _g):
        self.set_gravity(_g)

    def remove_all_constraints(self, ):
        papi.world__removeAllConstraints(self.id)

    def set_collision_detector(self, detector_type):
        """
        self.set_collision_detector(World.DART_COLLISION_DETECTOR)
        """
        papi.world__setCollisionDetector(self.id, detector_type)
        assert(detector_type == self.collision_detector())

    def collision_detector(self, ):
        return papi.world__getCollisionDetector(self.id)

    def collision_detector_string(self, ):
        typenames = ["DART_COLLISION_DETECTOR",
                     "FCL_COLLISION_DETECTOR",
                     "BULLET_COLLISION_DETECTOR", ]
        detector_type = self.collision_detector()
        return typenames[detector_type]

    # def set_collision_pair(self, body1, body2, is_enable):
    #     flag_enable = 1 if is_enable else 0
    #     papi.setWorldCollisionPair(self.id,
    #                                body1.skel.id, body1.id,
    #                                body2.skel.id, body2.id,
    #                                flag_enable)

    def __repr__(self):
        return "[World.%d %.4f]" % (self.id, self.t)



from __future__ import absolute_import
from builtins import range
from builtins import object
# Copyright (c) 2015, Disney Research
# All rights reserved.
#
# Author(s): Sehoon Ha <sehoon.ha@disneyresearch.com>
# Disney Research Robotics Group
from . import pydart2_api as papi


class Joint(object):
    """
    """
    FORCE, PASSIVE, SERVO, ACCELERATION, VELOCITY, LOCKED = list(range(6))

    def __init__(self, _skeleton, _id):
        """
        """
        self.skeleton = _skeleton
        self.id = _id
        self.name = papi.joint__getName(self.wid, self.skid, self.id)
        self.parent_bodynode = None
        self.child_bodynode = None
        self.dofs = list()

    @property
    def skel(self):
        return self.skeleton

    @property
    def wid(self):
        return self.skel.world.id

    @property
    def skid(self):
        return self.skel.id

    def build(self):
        skel = self.skel
        self.parent_bodynode = None
        self.child_bodynode = None

        parent_id = self.parent_body_node_id()
        if parent_id >= 0:
            self.parent_bodynode = skel.bodynodes[
                self.parent_body_node_id()]

        child_id = self.child_body_node_id()
        if child_id >= 0:
            self.child_bodynode = skel.bodynodes[
                self.child_body_node_id()]

        self.dofs = list()
        _ndofs = papi.joint__getNumDofs(self.wid, self.skid, self.id)
        for i in range(_ndofs):
            id = papi.joint__getDof(self.wid, self.skid, self.id, i)
            self.dofs.append(skel.dofs[id])
            skel.dofs[id].joint = self

########################################
# Joint::Property Functions
    def set_name(self, _name, _renameDofs):
        return papi.joint__setName(self.wid,
                                   self.skid,
                                   self.id,
                                   _name,
                                   _renameDofs)

    def is_kinematic(self, ):
        return papi.joint__isKinematic(self.wid, self.skid, self.id)

    def is_dynamic(self, ):
        return papi.joint__isDynamic(self.wid, self.skid, self.id)

    def type(self, ):
        return papi.joint__getType(self.wid, self.skid, self.id)

    def set_actuator_type(self, actuator_type):
        """
        joint.set_actuator_type(Joint.LOCKED)
        """
        papi.joint__setActuatorType(
            self.wid, self.skid, self.id, actuator_type)

    def actuator_type(self, ):
        return papi.joint__getActuatorType(self.wid, self.skid, self.id)

########################################
# Joint::Parent and child functions
    def position_in_world_frame(self, ):
        T0 = self.child_bodynode.transform()
        T1 = self.transform_from_child_body_node()
        T = T0.dot(T1)
        return T[:3, 3]

    def parent_body_node_id(self, ):
        return papi.joint__getParentBodyNode(self.wid, self.skid, self.id)

    def child_body_node_id(self, ):
        return papi.joint__getChildBodyNode(self.wid, self.skid, self.id)

    def set_transform_from_parent_body_node(self, T):
        papi.joint__setTransformFromParentBodyNode(self.wid,
                                                   self.skid,
                                                   self.id,
                                                   T)

    def set_transform_from_child_body_node(self, T):
        papi.joint__setTransformFromChildBodyNode(self.wid,
                                                  self.skid,
                                                  self.id,
                                                  T)

    def transform_from_parent_body_node(self, ):
        return papi.joint__getTransformFromParentBodyNode(self.wid,
                                                          self.skid,
                                                          self.id)

    def transform_from_child_body_node(self, ):
        return papi.joint__getTransformFromChildBodyNode(self.wid,
                                                         self.skid,
                                                         self.id)


########################################
# Joint::Limit functions
    def set_position_limit_enforced(self, _isPositionLimitEnforced=True):
        papi.joint__setPositionLimitEnforced(self.wid,
                                             self.skid,
                                             self.id,
                                             _isPositionLimitEnforced)

    def is_position_limit_enforced(self, ):
        return papi.joint__isPositionLimitEnforced(self.wid,
                                                   self.skid,
                                                   self.id)

    def has_position_limit(self, _index):
        return papi.joint__hasPositionLimit(self.wid,
                                            self.skid,
                                            self.id,
                                            _index)

    def position_lower_limit(self, _index):
        return papi.joint__getPositionLowerLimit(self.wid,
                                                 self.skid,
                                                 self.id,
                                                 _index)

    def set_position_lower_limit(self, _index, _position):
        papi.joint__setPositionLowerLimit(self.wid,
                                          self.skid,
                                          self.id,
                                          _index,
                                          _position)

    def position_upper_limit(self, _index):
        return papi.joint__getPositionUpperLimit(self.wid,
                                                 self.skid,
                                                 self.id,
                                                 _index)

    def set_position_upper_limit(self, _index, _position):
        papi.joint__setPositionUpperLimit(self.wid,
                                          self.skid,
                                          self.id,
                                          _index,
                                          _position)

########################################
# Joint::Dof functions
    def num_dofs(self, ):
        return len(self.dofs)

########################################
# Joint::Passive Force functions
    def spring_stiffness(self, _index):
        return papi.joint__getSpringStiffness(self.wid,
                                              self.skid,
                                              self.id,
                                              _index)

    def set_spring_stiffness(self, _index, _k):
        papi.joint__setSpringStiffness(self.wid,
                                       self.skid,
                                       self.id,
                                       _index,
                                       _k)

    def rest_position(self, _index):
        return papi.joint__getRestPosition(self.wid,
                                           self.skid,
                                           self.id,
                                           _index)

    def set_rest_position(self, _index, _q0):
        papi.joint__setRestPosition(self.wid, self.skid, self.id,
                                    _index,
                                    _q0)

    def damping_coefficient(self, _index):
        return papi.joint__getDampingCoefficient(self.wid,
                                                 self.skid,
                                                 self.id,
                                                 _index)

    def set_damping_coefficient(self, _index, _coeff):
        papi.joint__setDampingCoefficient(self.wid,
                                          self.skid,
                                          self.id,
                                          _index,
                                          _coeff)

    def coulomb_friction(self, _index):
        return papi.joint__getCoulombFriction(self.wid,
                                              self.skid,
                                              self.id,
                                              _index)

    def set_coulomb_friction(self, _index, _friction):
        papi.joint__setCoulombFriction(self.wid,
                                       self.skid,
                                       self.id,
                                       _index,
                                       _friction)

    def __repr__(self):
        return '[Joint(%d): %s]' % (self.id, self.name)


class WeldJoint(Joint):

    def __init__(self, _skeleton, _id):
        Joint.__init__(self, _skeleton, _id)

    def __repr__(self):
        return '[WeldJoint(%d): %s]' % (self.id, self.name)


class RevoluteJoint(Joint):

    def __init__(self, _skeleton, _id):
        Joint.__init__(self, _skeleton, _id)

    def axis(self, ):
        return papi.revolute_joint__getAxis(self.wid, self.skid, self.id)

    def axis_in_world_frame(self, ):
        R = self.parent_bodynode.T[:3, :3]
        return R.dot(self.axis())

    def set_axis(self, _axis):
        return papi.revolute_joint__setAxis(
            self.wid, self.skid, self.id, _axis)

    def __repr__(self):
        return '[RevoluteJoint(%d): %s]' % (self.id, self.name)


class PrismaticJoint(Joint):

    def __init__(self, _skeleton, _id):
        Joint.__init__(self, _skeleton, _id)

    def axis(self, ):
        return papi.prismatic_joint__getAxis(self.wid, self.skid, self.id)

    def set_axis(self, _axis):
        return papi.prismatic_joint__setAxis(
            self.wid, self.skid, self.id, _axis)

    def __repr__(self):
        return '[PrismaticJoint(%d): %s]' % (self.id, self.name)


class UniversalJoint(Joint):

    def __init__(self, _skeleton, _id):
        Joint.__init__(self, _skeleton, _id)

    def axis1(self, ):
        return papi.universal_joint__getAxis1(self.wid, self.skid, self.id)

    def axis1_in_world_frame(self, ):
        R = self.parent_bodynode.T[:3, :3]
        return R.dot(self.axis1())

    def set_axis1(self, _axis):
        return papi.universal_joint__setAxis1(
            self.wid, self.skid, self.id, _axis)

    def axis2_in_world_frame(self, ):
        R = self.parent_bodynode.T[:3, :3]
        return R.dot(self.axis2())

    def axis2(self, ):
        return papi.universal_joint__getAxis2(self.wid, self.skid, self.id)

    def set_axis2(self, _axis):
        return papi.universal_joint__setAxis2(
            self.wid, self.skid, self.id, _axis)

    def __repr__(self):
        return '[UniversalJoint(%d): %s]' % (self.id, self.name)


class PlanarJoint(Joint):

    def __init__(self, _skeleton, _id):
        Joint.__init__(self, _skeleton, _id)

    def __repr__(self):
        return '[PlanarJoint(%d): %s]' % (self.id, self.name)


class ScrewJoint(Joint):

    def __init__(self, _skeleton, _id):
        Joint.__init__(self, _skeleton, _id)

    def __repr__(self):
        return '[ScrewJoint(%d): %s]' % (self.id, self.name)


class BallJoint(Joint):

    def __init__(self, _skeleton, _id):
        Joint.__init__(self, _skeleton, _id)

    def __repr__(self):
        return '[BallJoint(%d): %s]' % (self.id, self.name)


class TranslationalJoint(Joint):

    def __init__(self, _skeleton, _id):
        Joint.__init__(self, _skeleton, _id)

    def __repr__(self):
        return '[TranslationalJoint(%d): %s]' % (self.id, self.name)


class EulerJoint(Joint):

    def __init__(self, _skeleton, _id):
        Joint.__init__(self, _skeleton, _id)

    def axis_order(self, ):
        return papi.euler_joint__getAxisOrder(self.wid, self.skid, self.id)

    def set_axis_order(self, _axis_order):
        return papi.euler_joint__setAxisOrder(
            self.wid, self.skid, self.id, _axis_order)

    def __repr__(self):
        return '[EulerJoint(%d): %s]' % (self.id, self.name)


class FreeJoint(Joint):

    def __init__(self, _skeleton, _id):
        Joint.__init__(self, _skeleton, _id)

    def __repr__(self):
        return '[FreeJoint(%d): %s]' % (self.id, self.name)


def create_joint(_skeleton, _id):
    wid = _skeleton.world.id
    skid = _skeleton.id
    id = _id
    type_str = papi.joint__getType(wid, skid, id)

    type_str_to_class = dict()
    type_str_to_class['WeldJoint'] = WeldJoint
    type_str_to_class['RevoluteJoint'] = RevoluteJoint
    type_str_to_class['PrismaticJoint'] = PrismaticJoint
    type_str_to_class['UniversalJoint'] = UniversalJoint
    type_str_to_class['PlanarJoint'] = PlanarJoint
    type_str_to_class['ScrewJoint'] = ScrewJoint
    type_str_to_class['BallJoint'] = BallJoint
    type_str_to_class['TranslationalJoint'] = TranslationalJoint
    type_str_to_class['EulerJoint'] = EulerJoint
    type_str_to_class['FreeJoint'] = FreeJoint

    cls = type_str_to_class.get(type_str, None)
    if cls is None:
        print("Invalid type string: %s" % type_str)
        return None
    else:
        return cls(_skeleton, _id)



# This file was automatically generated by SWIG (http://www.swig.org).
# Version 2.0.12
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.





from sys import version_info
if version_info >= (2,6,0):
    def swig_import_helper():
        from os.path import dirname
        import imp
        fp = None
        try:
            fp, pathname, description = imp.find_module('_pydart2_api', [dirname(__file__)])
        except ImportError:
            import _pydart2_api
            return _pydart2_api
        if fp is not None:
            try:
                _mod = imp.load_module('_pydart2_api', fp, pathname, description)
            finally:
                fp.close()
            return _mod
    _pydart2_api = swig_import_helper()
    del swig_import_helper
else:
    import _pydart2_api
del version_info
try:
    _swig_property = property
except NameError:
    pass # Python < 2.2 doesn't have 'property'.
def _swig_setattr_nondynamic(self,class_type,name,value,static=1):
    if (name == "thisown"): return self.this.own(value)
    if (name == "this"):
        if type(value).__name__ == 'SwigPyObject':
            self.__dict__[name] = value
            return
    method = class_type.__swig_setmethods__.get(name,None)
    if method: return method(self,value)
    if (not static):
        self.__dict__[name] = value
    else:
        raise AttributeError("You cannot add attributes to %s" % self)

def _swig_setattr(self,class_type,name,value):
    return _swig_setattr_nondynamic(self,class_type,name,value,0)

def _swig_getattr(self,class_type,name):
    if (name == "thisown"): return self.this.own()
    method = class_type.__swig_getmethods__.get(name,None)
    if method: return method(self)
    raise AttributeError(name)

def _swig_repr(self):
    try: strthis = "proxy of " + self.this.__repr__()
    except: strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)

try:
    _object = object
    _newclass = 1
except AttributeError:
    class _object : pass
    _newclass = 0


class SwigPyIterator(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, SwigPyIterator, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, SwigPyIterator, name)
    def __init__(self, *args, **kwargs): raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _pydart2_api.delete_SwigPyIterator
    __del__ = lambda self : None;
    def value(self) -> "PyObject *" : return _pydart2_api.SwigPyIterator_value(self)
    def incr(self, n : 'size_t'=1) -> "swig::SwigPyIterator *" : return _pydart2_api.SwigPyIterator_incr(self, n)
    def decr(self, n : 'size_t'=1) -> "swig::SwigPyIterator *" : return _pydart2_api.SwigPyIterator_decr(self, n)
    def distance(self, *args) -> "ptrdiff_t" : return _pydart2_api.SwigPyIterator_distance(self, *args)
    def equal(self, *args) -> "bool" : return _pydart2_api.SwigPyIterator_equal(self, *args)
    def copy(self) -> "swig::SwigPyIterator *" : return _pydart2_api.SwigPyIterator_copy(self)
    def next(self) -> "PyObject *" : return _pydart2_api.SwigPyIterator_next(self)
    def __next__(self) -> "PyObject *" : return _pydart2_api.SwigPyIterator___next__(self)
    def previous(self) -> "PyObject *" : return _pydart2_api.SwigPyIterator_previous(self)
    def advance(self, *args) -> "swig::SwigPyIterator *" : return _pydart2_api.SwigPyIterator_advance(self, *args)
    def __eq__(self, *args) -> "bool" : return _pydart2_api.SwigPyIterator___eq__(self, *args)
    def __ne__(self, *args) -> "bool" : return _pydart2_api.SwigPyIterator___ne__(self, *args)
    def __iadd__(self, *args) -> "swig::SwigPyIterator &" : return _pydart2_api.SwigPyIterator___iadd__(self, *args)
    def __isub__(self, *args) -> "swig::SwigPyIterator &" : return _pydart2_api.SwigPyIterator___isub__(self, *args)
    def __add__(self, *args) -> "swig::SwigPyIterator *" : return _pydart2_api.SwigPyIterator___add__(self, *args)
    def __sub__(self, *args) -> "ptrdiff_t" : return _pydart2_api.SwigPyIterator___sub__(self, *args)
    def __iter__(self): return self
SwigPyIterator_swigregister = _pydart2_api.SwigPyIterator_swigregister
SwigPyIterator_swigregister(SwigPyIterator)

class IntVector(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, IntVector, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, IntVector, name)
    __repr__ = _swig_repr
    def iterator(self) -> "swig::SwigPyIterator *" : return _pydart2_api.IntVector_iterator(self)
    def __iter__(self): return self.iterator()
    def __nonzero__(self) -> "bool" : return _pydart2_api.IntVector___nonzero__(self)
    def __bool__(self) -> "bool" : return _pydart2_api.IntVector___bool__(self)
    def __len__(self) -> "std::vector< int >::size_type" : return _pydart2_api.IntVector___len__(self)
    def pop(self) -> "std::vector< int >::value_type" : return _pydart2_api.IntVector_pop(self)
    def __getslice__(self, *args) -> "std::vector< int,std::allocator< int > > *" : return _pydart2_api.IntVector___getslice__(self, *args)
    def __setslice__(self, *args) -> "void" : return _pydart2_api.IntVector___setslice__(self, *args)
    def __delslice__(self, *args) -> "void" : return _pydart2_api.IntVector___delslice__(self, *args)
    def __delitem__(self, *args) -> "void" : return _pydart2_api.IntVector___delitem__(self, *args)
    def __getitem__(self, *args) -> "std::vector< int >::value_type const &" : return _pydart2_api.IntVector___getitem__(self, *args)
    def __setitem__(self, *args) -> "void" : return _pydart2_api.IntVector___setitem__(self, *args)
    def append(self, *args) -> "void" : return _pydart2_api.IntVector_append(self, *args)
    def empty(self) -> "bool" : return _pydart2_api.IntVector_empty(self)
    def size(self) -> "std::vector< int >::size_type" : return _pydart2_api.IntVector_size(self)
    def clear(self) -> "void" : return _pydart2_api.IntVector_clear(self)
    def swap(self, *args) -> "void" : return _pydart2_api.IntVector_swap(self, *args)
    def get_allocator(self) -> "std::vector< int >::allocator_type" : return _pydart2_api.IntVector_get_allocator(self)
    def begin(self) -> "std::vector< int >::iterator" : return _pydart2_api.IntVector_begin(self)
    def end(self) -> "std::vector< int >::iterator" : return _pydart2_api.IntVector_end(self)
    def rbegin(self) -> "std::vector< int >::reverse_iterator" : return _pydart2_api.IntVector_rbegin(self)
    def rend(self) -> "std::vector< int >::reverse_iterator" : return _pydart2_api.IntVector_rend(self)
    def pop_back(self) -> "void" : return _pydart2_api.IntVector_pop_back(self)
    def erase(self, *args) -> "std::vector< int >::iterator" : return _pydart2_api.IntVector_erase(self, *args)
    def __init__(self, *args): 
        this = _pydart2_api.new_IntVector(*args)
        try: self.this.append(this)
        except: self.this = this
    def push_back(self, *args) -> "void" : return _pydart2_api.IntVector_push_back(self, *args)
    def front(self) -> "std::vector< int >::value_type const &" : return _pydart2_api.IntVector_front(self)
    def back(self) -> "std::vector< int >::value_type const &" : return _pydart2_api.IntVector_back(self)
    def assign(self, *args) -> "void" : return _pydart2_api.IntVector_assign(self, *args)
    def resize(self, *args) -> "void" : return _pydart2_api.IntVector_resize(self, *args)
    def insert(self, *args) -> "void" : return _pydart2_api.IntVector_insert(self, *args)
    def reserve(self, *args) -> "void" : return _pydart2_api.IntVector_reserve(self, *args)
    def capacity(self) -> "std::vector< int >::size_type" : return _pydart2_api.IntVector_capacity(self)
    __swig_destroy__ = _pydart2_api.delete_IntVector
    __del__ = lambda self : None;
IntVector_swigregister = _pydart2_api.IntVector_swigregister
IntVector_swigregister(IntVector)


def init(verbose : 'bool'=True) -> "void" :
  return _pydart2_api.init(verbose)
init = _pydart2_api.init

def destroy() -> "void" :
  return _pydart2_api.destroy()
destroy = _pydart2_api.destroy

def setVerbose(verbose : 'bool'=True) -> "void" :
  return _pydart2_api.setVerbose(verbose)
setVerbose = _pydart2_api.setVerbose

def getVerbose() -> "bool" :
  return _pydart2_api.getVerbose()
getVerbose = _pydart2_api.getVerbose

def marker__getBodyNode(*args) -> "int" :
  return _pydart2_api.marker__getBodyNode(*args)
marker__getBodyNode = _pydart2_api.marker__getBodyNode

def marker__getLocalPosition(*args) -> "double [3]" :
  return _pydart2_api.marker__getLocalPosition(*args)
marker__getLocalPosition = _pydart2_api.marker__getLocalPosition

def marker__setLocalPosition(*args) -> "void" :
  return _pydart2_api.marker__setLocalPosition(*args)
marker__setLocalPosition = _pydart2_api.marker__setLocalPosition

def marker__getWorldPosition(*args) -> "double [3]" :
  return _pydart2_api.marker__getWorldPosition(*args)
marker__getWorldPosition = _pydart2_api.marker__getWorldPosition

def marker__render(*args) -> "void" :
  return _pydart2_api.marker__render(*args)
marker__render = _pydart2_api.marker__render

def collisionresult__getNumContacts(*args) -> "int" :
  return _pydart2_api.collisionresult__getNumContacts(*args)
collisionresult__getNumContacts = _pydart2_api.collisionresult__getNumContacts

def collisionresult__getContacts(*args) -> "int" :
  return _pydart2_api.collisionresult__getContacts(*args)
collisionresult__getContacts = _pydart2_api.collisionresult__getContacts

def collisionresult__getCollidingBodyNodes(*args) -> "std::vector< int,std::allocator< int > >" :
  return _pydart2_api.collisionresult__getCollidingBodyNodes(*args)
collisionresult__getCollidingBodyNodes = _pydart2_api.collisionresult__getCollidingBodyNodes

def collisionresult__renderContact(*args) -> "void" :
  return _pydart2_api.collisionresult__renderContact(*args)
collisionresult__renderContact = _pydart2_api.collisionresult__renderContact

def addBallJointConstraint(*args) -> "int" :
  return _pydart2_api.addBallJointConstraint(*args)
addBallJointConstraint = _pydart2_api.addBallJointConstraint

def createWorld(*args) -> "int" :
  return _pydart2_api.createWorld(*args)
createWorld = _pydart2_api.createWorld

def createWorldFromSkel(*args) -> "int" :
  return _pydart2_api.createWorldFromSkel(*args)
createWorldFromSkel = _pydart2_api.createWorldFromSkel

def destroyWorld(*args) -> "void" :
  return _pydart2_api.destroyWorld(*args)
destroyWorld = _pydart2_api.destroyWorld

def world__addSkeleton(*args) -> "int" :
  return _pydart2_api.world__addSkeleton(*args)
world__addSkeleton = _pydart2_api.world__addSkeleton

def world__getNumSkeletons(*args) -> "int" :
  return _pydart2_api.world__getNumSkeletons(*args)
world__getNumSkeletons = _pydart2_api.world__getNumSkeletons

def world__reset(*args) -> "void" :
  return _pydart2_api.world__reset(*args)
world__reset = _pydart2_api.world__reset

def world__step(*args) -> "void" :
  return _pydart2_api.world__step(*args)
world__step = _pydart2_api.world__step

def world__checkCollision(*args) -> "void" :
  return _pydart2_api.world__checkCollision(*args)
world__checkCollision = _pydart2_api.world__checkCollision

def world__render(*args) -> "void" :
  return _pydart2_api.world__render(*args)
world__render = _pydart2_api.world__render

def world__setTimeStep(*args) -> "void" :
  return _pydart2_api.world__setTimeStep(*args)
world__setTimeStep = _pydart2_api.world__setTimeStep

def world__getTimeStep(*args) -> "double" :
  return _pydart2_api.world__getTimeStep(*args)
world__getTimeStep = _pydart2_api.world__getTimeStep

def world__setTime(*args) -> "void" :
  return _pydart2_api.world__setTime(*args)
world__setTime = _pydart2_api.world__setTime

def world__getTime(*args) -> "double" :
  return _pydart2_api.world__getTime(*args)
world__getTime = _pydart2_api.world__getTime

def world__getSimFrames(*args) -> "int" :
  return _pydart2_api.world__getSimFrames(*args)
world__getSimFrames = _pydart2_api.world__getSimFrames

def world__getIndex(*args) -> "int" :
  return _pydart2_api.world__getIndex(*args)
world__getIndex = _pydart2_api.world__getIndex

def world__setGravity(*args) -> "void" :
  return _pydart2_api.world__setGravity(*args)
world__setGravity = _pydart2_api.world__setGravity

def world__getGravity(*args) -> "double [3]" :
  return _pydart2_api.world__getGravity(*args)
world__getGravity = _pydart2_api.world__getGravity

def world__setCollisionDetector(*args) -> "void" :
  return _pydart2_api.world__setCollisionDetector(*args)
world__setCollisionDetector = _pydart2_api.world__setCollisionDetector

def world__getCollisionDetector(*args) -> "int" :
  return _pydart2_api.world__getCollisionDetector(*args)
world__getCollisionDetector = _pydart2_api.world__getCollisionDetector

def world__removeAllConstraints(*args) -> "void" :
  return _pydart2_api.world__removeAllConstraints(*args)
world__removeAllConstraints = _pydart2_api.world__removeAllConstraints

def skeleton__render(*args) -> "void" :
  return _pydart2_api.skeleton__render(*args)
skeleton__render = _pydart2_api.skeleton__render

def skeleton__renderWithColor(*args) -> "void" :
  return _pydart2_api.skeleton__renderWithColor(*args)
skeleton__renderWithColor = _pydart2_api.skeleton__renderWithColor

def skeleton__getName(*args) -> "char const *" :
  return _pydart2_api.skeleton__getName(*args)
skeleton__getName = _pydart2_api.skeleton__getName

def skeleton__getMass(*args) -> "double" :
  return _pydart2_api.skeleton__getMass(*args)
skeleton__getMass = _pydart2_api.skeleton__getMass

def skeleton__isMobile(*args) -> "bool" :
  return _pydart2_api.skeleton__isMobile(*args)
skeleton__isMobile = _pydart2_api.skeleton__isMobile

def skeleton__setMobile(*args) -> "void" :
  return _pydart2_api.skeleton__setMobile(*args)
skeleton__setMobile = _pydart2_api.skeleton__setMobile

def skeleton__getSelfCollisionCheck(*args) -> "bool" :
  return _pydart2_api.skeleton__getSelfCollisionCheck(*args)
skeleton__getSelfCollisionCheck = _pydart2_api.skeleton__getSelfCollisionCheck

def skeleton__setSelfCollisionCheck(*args) -> "void" :
  return _pydart2_api.skeleton__setSelfCollisionCheck(*args)
skeleton__setSelfCollisionCheck = _pydart2_api.skeleton__setSelfCollisionCheck

def skeleton__getAdjacentBodyCheck(*args) -> "bool" :
  return _pydart2_api.skeleton__getAdjacentBodyCheck(*args)
skeleton__getAdjacentBodyCheck = _pydart2_api.skeleton__getAdjacentBodyCheck

def skeleton__setAdjacentBodyCheck(*args) -> "void" :
  return _pydart2_api.skeleton__setAdjacentBodyCheck(*args)
skeleton__setAdjacentBodyCheck = _pydart2_api.skeleton__setAdjacentBodyCheck

def skeleton__setRootJointToTransAndEuler(*args) -> "void" :
  return _pydart2_api.skeleton__setRootJointToTransAndEuler(*args)
skeleton__setRootJointToTransAndEuler = _pydart2_api.skeleton__setRootJointToTransAndEuler

def skeleton__getNumBodyNodes(*args) -> "int" :
  return _pydart2_api.skeleton__getNumBodyNodes(*args)
skeleton__getNumBodyNodes = _pydart2_api.skeleton__getNumBodyNodes

def skeleton__getNumJoints(*args) -> "int" :
  return _pydart2_api.skeleton__getNumJoints(*args)
skeleton__getNumJoints = _pydart2_api.skeleton__getNumJoints

def skeleton__getNumDofs(*args) -> "int" :
  return _pydart2_api.skeleton__getNumDofs(*args)
skeleton__getNumDofs = _pydart2_api.skeleton__getNumDofs

def skeleton__getNumMarkers(*args) -> "int" :
  return _pydart2_api.skeleton__getNumMarkers(*args)
skeleton__getNumMarkers = _pydart2_api.skeleton__getNumMarkers

def skeleton__getPositions(*args) -> "int" :
  return _pydart2_api.skeleton__getPositions(*args)
skeleton__getPositions = _pydart2_api.skeleton__getPositions

def skeleton__setPositions(*args) -> "void" :
  return _pydart2_api.skeleton__setPositions(*args)
skeleton__setPositions = _pydart2_api.skeleton__setPositions

def skeleton__getVelocities(*args) -> "int" :
  return _pydart2_api.skeleton__getVelocities(*args)
skeleton__getVelocities = _pydart2_api.skeleton__getVelocities

def skeleton__setVelocities(*args) -> "void" :
  return _pydart2_api.skeleton__setVelocities(*args)
skeleton__setVelocities = _pydart2_api.skeleton__setVelocities

def skeleton__getAccelerations(*args) -> "int" :
  return _pydart2_api.skeleton__getAccelerations(*args)
skeleton__getAccelerations = _pydart2_api.skeleton__getAccelerations

def skeleton__setForces(*args) -> "void" :
  return _pydart2_api.skeleton__setForces(*args)
skeleton__setForces = _pydart2_api.skeleton__setForces

def skeleton__getPositionDifferences(*args) -> "int" :
  return _pydart2_api.skeleton__getPositionDifferences(*args)
skeleton__getPositionDifferences = _pydart2_api.skeleton__getPositionDifferences

def skeleton__getVelocityDifferences(*args) -> "int" :
  return _pydart2_api.skeleton__getVelocityDifferences(*args)
skeleton__getVelocityDifferences = _pydart2_api.skeleton__getVelocityDifferences

def skeleton__getPositionLowerLimits(*args) -> "int" :
  return _pydart2_api.skeleton__getPositionLowerLimits(*args)
skeleton__getPositionLowerLimits = _pydart2_api.skeleton__getPositionLowerLimits

def skeleton__getPositionUpperLimits(*args) -> "int" :
  return _pydart2_api.skeleton__getPositionUpperLimits(*args)
skeleton__getPositionUpperLimits = _pydart2_api.skeleton__getPositionUpperLimits

def skeleton__getForceLowerLimits(*args) -> "int" :
  return _pydart2_api.skeleton__getForceLowerLimits(*args)
skeleton__getForceLowerLimits = _pydart2_api.skeleton__getForceLowerLimits

def skeleton__getForceUpperLimits(*args) -> "int" :
  return _pydart2_api.skeleton__getForceUpperLimits(*args)
skeleton__getForceUpperLimits = _pydart2_api.skeleton__getForceUpperLimits

def skeleton__getCOM(*args) -> "double [3]" :
  return _pydart2_api.skeleton__getCOM(*args)
skeleton__getCOM = _pydart2_api.skeleton__getCOM

def skeleton__getCOMLinearVelocity(*args) -> "double [3]" :
  return _pydart2_api.skeleton__getCOMLinearVelocity(*args)
skeleton__getCOMLinearVelocity = _pydart2_api.skeleton__getCOMLinearVelocity

def skeleton__getCOMLinearAcceleration(*args) -> "double [3]" :
  return _pydart2_api.skeleton__getCOMLinearAcceleration(*args)
skeleton__getCOMLinearAcceleration = _pydart2_api.skeleton__getCOMLinearAcceleration

def skeleton__getMassMatrix(*args) -> "void" :
  return _pydart2_api.skeleton__getMassMatrix(*args)
skeleton__getMassMatrix = _pydart2_api.skeleton__getMassMatrix

def skeleton__getCoriolisAndGravityForces(*args) -> "int" :
  return _pydart2_api.skeleton__getCoriolisAndGravityForces(*args)
skeleton__getCoriolisAndGravityForces = _pydart2_api.skeleton__getCoriolisAndGravityForces

def skeleton__getConstraintForces(*args) -> "int" :
  return _pydart2_api.skeleton__getConstraintForces(*args)
skeleton__getConstraintForces = _pydart2_api.skeleton__getConstraintForces

def bodynode__getName(*args) -> "char const *" :
  return _pydart2_api.bodynode__getName(*args)
bodynode__getName = _pydart2_api.bodynode__getName

def bodynode__getParentBodyNode(*args) -> "int" :
  return _pydart2_api.bodynode__getParentBodyNode(*args)
bodynode__getParentBodyNode = _pydart2_api.bodynode__getParentBodyNode

def bodynode__getNumChildBodyNodes(*args) -> "int" :
  return _pydart2_api.bodynode__getNumChildBodyNodes(*args)
bodynode__getNumChildBodyNodes = _pydart2_api.bodynode__getNumChildBodyNodes

def bodynode__getChildBodyNode(*args) -> "int" :
  return _pydart2_api.bodynode__getChildBodyNode(*args)
bodynode__getChildBodyNode = _pydart2_api.bodynode__getChildBodyNode

def bodynode__getParentJoint(*args) -> "int" :
  return _pydart2_api.bodynode__getParentJoint(*args)
bodynode__getParentJoint = _pydart2_api.bodynode__getParentJoint

def bodynode__getNumChildJoints(*args) -> "int" :
  return _pydart2_api.bodynode__getNumChildJoints(*args)
bodynode__getNumChildJoints = _pydart2_api.bodynode__getNumChildJoints

def bodynode__getChildJoint(*args) -> "int" :
  return _pydart2_api.bodynode__getChildJoint(*args)
bodynode__getChildJoint = _pydart2_api.bodynode__getChildJoint

def bodynode__getNumDependentDofs(*args) -> "int" :
  return _pydart2_api.bodynode__getNumDependentDofs(*args)
bodynode__getNumDependentDofs = _pydart2_api.bodynode__getNumDependentDofs

def bodynode__getDependentDof(*args) -> "int" :
  return _pydart2_api.bodynode__getDependentDof(*args)
bodynode__getDependentDof = _pydart2_api.bodynode__getDependentDof

def bodynode__getNumShapeNodes(*args) -> "int" :
  return _pydart2_api.bodynode__getNumShapeNodes(*args)
bodynode__getNumShapeNodes = _pydart2_api.bodynode__getNumShapeNodes

def bodynode__getIndexInSkeleton(*args) -> "int" :
  return _pydart2_api.bodynode__getIndexInSkeleton(*args)
bodynode__getIndexInSkeleton = _pydart2_api.bodynode__getIndexInSkeleton

def bodynode__getIndexInTree(*args) -> "int" :
  return _pydart2_api.bodynode__getIndexInTree(*args)
bodynode__getIndexInTree = _pydart2_api.bodynode__getIndexInTree

def bodynode__getTreeIndex(*args) -> "int" :
  return _pydart2_api.bodynode__getTreeIndex(*args)
bodynode__getTreeIndex = _pydart2_api.bodynode__getTreeIndex

def bodynode__setGravityMode(*args) -> "void" :
  return _pydart2_api.bodynode__setGravityMode(*args)
bodynode__setGravityMode = _pydart2_api.bodynode__setGravityMode

def bodynode__getGravityMode(*args) -> "bool" :
  return _pydart2_api.bodynode__getGravityMode(*args)
bodynode__getGravityMode = _pydart2_api.bodynode__getGravityMode

def bodynode__isCollidable(*args) -> "bool" :
  return _pydart2_api.bodynode__isCollidable(*args)
bodynode__isCollidable = _pydart2_api.bodynode__isCollidable

def bodynode__setCollidable(*args) -> "void" :
  return _pydart2_api.bodynode__setCollidable(*args)
bodynode__setCollidable = _pydart2_api.bodynode__setCollidable

def bodynode__getMass(*args) -> "double" :
  return _pydart2_api.bodynode__getMass(*args)
bodynode__getMass = _pydart2_api.bodynode__getMass

def bodynode__getInertia(*args) -> "double [3][3]" :
  return _pydart2_api.bodynode__getInertia(*args)
bodynode__getInertia = _pydart2_api.bodynode__getInertia

def bodynode__getLocalCOM(*args) -> "double [3]" :
  return _pydart2_api.bodynode__getLocalCOM(*args)
bodynode__getLocalCOM = _pydart2_api.bodynode__getLocalCOM

def bodynode__getCOM(*args) -> "double [3]" :
  return _pydart2_api.bodynode__getCOM(*args)
bodynode__getCOM = _pydart2_api.bodynode__getCOM

def bodynode__getCOMLinearVelocity(*args) -> "double [3]" :
  return _pydart2_api.bodynode__getCOMLinearVelocity(*args)
bodynode__getCOMLinearVelocity = _pydart2_api.bodynode__getCOMLinearVelocity

def bodynode__getCOMSpatialVelocity(*args) -> "double [6]" :
  return _pydart2_api.bodynode__getCOMSpatialVelocity(*args)
bodynode__getCOMSpatialVelocity = _pydart2_api.bodynode__getCOMSpatialVelocity

def bodynode__getCOMLinearAcceleration(*args) -> "double [3]" :
  return _pydart2_api.bodynode__getCOMLinearAcceleration(*args)
bodynode__getCOMLinearAcceleration = _pydart2_api.bodynode__getCOMLinearAcceleration

def bodynode__getCOMSpatialAcceleration(*args) -> "double [6]" :
  return _pydart2_api.bodynode__getCOMSpatialAcceleration(*args)
bodynode__getCOMSpatialAcceleration = _pydart2_api.bodynode__getCOMSpatialAcceleration

def bodynode__setFrictionCoeff(*args) -> "void" :
  return _pydart2_api.bodynode__setFrictionCoeff(*args)
bodynode__setFrictionCoeff = _pydart2_api.bodynode__setFrictionCoeff

def bodynode__getFrictionCoeff(*args) -> "double" :
  return _pydart2_api.bodynode__getFrictionCoeff(*args)
bodynode__getFrictionCoeff = _pydart2_api.bodynode__getFrictionCoeff

def bodynode__setRestitutionCoeff(*args) -> "void" :
  return _pydart2_api.bodynode__setRestitutionCoeff(*args)
bodynode__setRestitutionCoeff = _pydart2_api.bodynode__setRestitutionCoeff

def bodynode__getRestitutionCoeff(*args) -> "double" :
  return _pydart2_api.bodynode__getRestitutionCoeff(*args)
bodynode__getRestitutionCoeff = _pydart2_api.bodynode__getRestitutionCoeff

def bodynode__getTransform(*args) -> "double [4][4]" :
  return _pydart2_api.bodynode__getTransform(*args)
bodynode__getTransform = _pydart2_api.bodynode__getTransform

def bodynode__getWorldTransform(*args) -> "double [4][4]" :
  return _pydart2_api.bodynode__getWorldTransform(*args)
bodynode__getWorldTransform = _pydart2_api.bodynode__getWorldTransform

def bodynode__getRelativeTransform(*args) -> "double [4][4]" :
  return _pydart2_api.bodynode__getRelativeTransform(*args)
bodynode__getRelativeTransform = _pydart2_api.bodynode__getRelativeTransform

def bodynode__addExtForce(*args) -> "void" :
  return _pydart2_api.bodynode__addExtForce(*args)
bodynode__addExtForce = _pydart2_api.bodynode__addExtForce

def bodynode__setExtForce(*args) -> "void" :
  return _pydart2_api.bodynode__setExtForce(*args)
bodynode__setExtForce = _pydart2_api.bodynode__setExtForce

def bodynode__addExtTorque(*args) -> "void" :
  return _pydart2_api.bodynode__addExtTorque(*args)
bodynode__addExtTorque = _pydart2_api.bodynode__addExtTorque

def bodynode__setExtTorque(*args) -> "void" :
  return _pydart2_api.bodynode__setExtTorque(*args)
bodynode__setExtTorque = _pydart2_api.bodynode__setExtTorque

def bodynode__getJacobian(*args) -> "void" :
  return _pydart2_api.bodynode__getJacobian(*args)
bodynode__getJacobian = _pydart2_api.bodynode__getJacobian

def bodynode__getLinearJacobian(*args) -> "void" :
  return _pydart2_api.bodynode__getLinearJacobian(*args)
bodynode__getLinearJacobian = _pydart2_api.bodynode__getLinearJacobian

def bodynode__getAngularJacobian(*args) -> "void" :
  return _pydart2_api.bodynode__getAngularJacobian(*args)
bodynode__getAngularJacobian = _pydart2_api.bodynode__getAngularJacobian

def bodynode__getWorldJacobian(*args) -> "void" :
  return _pydart2_api.bodynode__getWorldJacobian(*args)
bodynode__getWorldJacobian = _pydart2_api.bodynode__getWorldJacobian

def bodynode__getLinearJacobianDeriv(*args) -> "void" :
  return _pydart2_api.bodynode__getLinearJacobianDeriv(*args)
bodynode__getLinearJacobianDeriv = _pydart2_api.bodynode__getLinearJacobianDeriv

def bodynode__getAngularJacobianDeriv(*args) -> "void" :
  return _pydart2_api.bodynode__getAngularJacobianDeriv(*args)
bodynode__getAngularJacobianDeriv = _pydart2_api.bodynode__getAngularJacobianDeriv

def dof__getName(*args) -> "char const *" :
  return _pydart2_api.dof__getName(*args)
dof__getName = _pydart2_api.dof__getName

def dof__getIndexInSkeleton(*args) -> "int" :
  return _pydart2_api.dof__getIndexInSkeleton(*args)
dof__getIndexInSkeleton = _pydart2_api.dof__getIndexInSkeleton

def dof__getIndexInTree(*args) -> "int" :
  return _pydart2_api.dof__getIndexInTree(*args)
dof__getIndexInTree = _pydart2_api.dof__getIndexInTree

def dof__getIndexInJoint(*args) -> "int" :
  return _pydart2_api.dof__getIndexInJoint(*args)
dof__getIndexInJoint = _pydart2_api.dof__getIndexInJoint

def dof__getTreeIndex(*args) -> "int" :
  return _pydart2_api.dof__getTreeIndex(*args)
dof__getTreeIndex = _pydart2_api.dof__getTreeIndex

def dof__getPosition(*args) -> "double" :
  return _pydart2_api.dof__getPosition(*args)
dof__getPosition = _pydart2_api.dof__getPosition

def dof__setPosition(*args) -> "void" :
  return _pydart2_api.dof__setPosition(*args)
dof__setPosition = _pydart2_api.dof__setPosition

def dof__getInitialPosition(*args) -> "double" :
  return _pydart2_api.dof__getInitialPosition(*args)
dof__getInitialPosition = _pydart2_api.dof__getInitialPosition

def dof__setInitialPosition(*args) -> "void" :
  return _pydart2_api.dof__setInitialPosition(*args)
dof__setInitialPosition = _pydart2_api.dof__setInitialPosition

def dof__hasPositionLimit(*args) -> "bool" :
  return _pydart2_api.dof__hasPositionLimit(*args)
dof__hasPositionLimit = _pydart2_api.dof__hasPositionLimit

def dof__getPositionLowerLimit(*args) -> "double" :
  return _pydart2_api.dof__getPositionLowerLimit(*args)
dof__getPositionLowerLimit = _pydart2_api.dof__getPositionLowerLimit

def dof__setPositionLowerLimit(*args) -> "void" :
  return _pydart2_api.dof__setPositionLowerLimit(*args)
dof__setPositionLowerLimit = _pydart2_api.dof__setPositionLowerLimit

def dof__getPositionUpperLimit(*args) -> "double" :
  return _pydart2_api.dof__getPositionUpperLimit(*args)
dof__getPositionUpperLimit = _pydart2_api.dof__getPositionUpperLimit

def dof__setPositionUpperLimit(*args) -> "void" :
  return _pydart2_api.dof__setPositionUpperLimit(*args)
dof__setPositionUpperLimit = _pydart2_api.dof__setPositionUpperLimit

def dof__getVelocity(*args) -> "double" :
  return _pydart2_api.dof__getVelocity(*args)
dof__getVelocity = _pydart2_api.dof__getVelocity

def dof__setVelocity(*args) -> "void" :
  return _pydart2_api.dof__setVelocity(*args)
dof__setVelocity = _pydart2_api.dof__setVelocity

def dof__getInitialVelocity(*args) -> "double" :
  return _pydart2_api.dof__getInitialVelocity(*args)
dof__getInitialVelocity = _pydart2_api.dof__getInitialVelocity

def dof__setInitialVelocity(*args) -> "void" :
  return _pydart2_api.dof__setInitialVelocity(*args)
dof__setInitialVelocity = _pydart2_api.dof__setInitialVelocity

def dof__getVelocityLowerLimit(*args) -> "double" :
  return _pydart2_api.dof__getVelocityLowerLimit(*args)
dof__getVelocityLowerLimit = _pydart2_api.dof__getVelocityLowerLimit

def dof__setVelocityLowerLimit(*args) -> "void" :
  return _pydart2_api.dof__setVelocityLowerLimit(*args)
dof__setVelocityLowerLimit = _pydart2_api.dof__setVelocityLowerLimit

def dof__getVelocityUpperLimit(*args) -> "double" :
  return _pydart2_api.dof__getVelocityUpperLimit(*args)
dof__getVelocityUpperLimit = _pydart2_api.dof__getVelocityUpperLimit

def dof__setVelocityUpperLimit(*args) -> "void" :
  return _pydart2_api.dof__setVelocityUpperLimit(*args)
dof__setVelocityUpperLimit = _pydart2_api.dof__setVelocityUpperLimit

def dof__getSpringStiffness(*args) -> "double" :
  return _pydart2_api.dof__getSpringStiffness(*args)
dof__getSpringStiffness = _pydart2_api.dof__getSpringStiffness

def dof__setSpringStiffness(*args) -> "void" :
  return _pydart2_api.dof__setSpringStiffness(*args)
dof__setSpringStiffness = _pydart2_api.dof__setSpringStiffness

def dof__getRestPosition(*args) -> "double" :
  return _pydart2_api.dof__getRestPosition(*args)
dof__getRestPosition = _pydart2_api.dof__getRestPosition

def dof__setRestPosition(*args) -> "void" :
  return _pydart2_api.dof__setRestPosition(*args)
dof__setRestPosition = _pydart2_api.dof__setRestPosition

def dof__getDampingCoefficient(*args) -> "double" :
  return _pydart2_api.dof__getDampingCoefficient(*args)
dof__getDampingCoefficient = _pydart2_api.dof__getDampingCoefficient

def dof__setDampingCoefficient(*args) -> "void" :
  return _pydart2_api.dof__setDampingCoefficient(*args)
dof__setDampingCoefficient = _pydart2_api.dof__setDampingCoefficient

def dof__getCoulombFriction(*args) -> "double" :
  return _pydart2_api.dof__getCoulombFriction(*args)
dof__getCoulombFriction = _pydart2_api.dof__getCoulombFriction

def dof__setCoulombFriction(*args) -> "void" :
  return _pydart2_api.dof__setCoulombFriction(*args)
dof__setCoulombFriction = _pydart2_api.dof__setCoulombFriction

def dof__getConstraintImpulse(*args) -> "double" :
  return _pydart2_api.dof__getConstraintImpulse(*args)
dof__getConstraintImpulse = _pydart2_api.dof__getConstraintImpulse

def dof__setConstraintImpulse(*args) -> "void" :
  return _pydart2_api.dof__setConstraintImpulse(*args)
dof__setConstraintImpulse = _pydart2_api.dof__setConstraintImpulse

def joint__getName(*args) -> "char const *" :
  return _pydart2_api.joint__getName(*args)
joint__getName = _pydart2_api.joint__getName

def joint__setName(*args) -> "char const *" :
  return _pydart2_api.joint__setName(*args)
joint__setName = _pydart2_api.joint__setName

def joint__isKinematic(*args) -> "bool" :
  return _pydart2_api.joint__isKinematic(*args)
joint__isKinematic = _pydart2_api.joint__isKinematic

def joint__isDynamic(*args) -> "bool" :
  return _pydart2_api.joint__isDynamic(*args)
joint__isDynamic = _pydart2_api.joint__isDynamic

def joint__getType(*args) -> "char const *" :
  return _pydart2_api.joint__getType(*args)
joint__getType = _pydart2_api.joint__getType

def joint__setActuatorType(*args) -> "void" :
  return _pydart2_api.joint__setActuatorType(*args)
joint__setActuatorType = _pydart2_api.joint__setActuatorType

def joint__getActuatorType(*args) -> "int" :
  return _pydart2_api.joint__getActuatorType(*args)
joint__getActuatorType = _pydart2_api.joint__getActuatorType

def joint__getParentBodyNode(*args) -> "int" :
  return _pydart2_api.joint__getParentBodyNode(*args)
joint__getParentBodyNode = _pydart2_api.joint__getParentBodyNode

def joint__getChildBodyNode(*args) -> "int" :
  return _pydart2_api.joint__getChildBodyNode(*args)
joint__getChildBodyNode = _pydart2_api.joint__getChildBodyNode

def joint__setTransformFromParentBodyNode(*args) -> "void" :
  return _pydart2_api.joint__setTransformFromParentBodyNode(*args)
joint__setTransformFromParentBodyNode = _pydart2_api.joint__setTransformFromParentBodyNode

def joint__setTransformFromChildBodyNode(*args) -> "void" :
  return _pydart2_api.joint__setTransformFromChildBodyNode(*args)
joint__setTransformFromChildBodyNode = _pydart2_api.joint__setTransformFromChildBodyNode

def joint__getTransformFromParentBodyNode(*args) -> "double [4][4]" :
  return _pydart2_api.joint__getTransformFromParentBodyNode(*args)
joint__getTransformFromParentBodyNode = _pydart2_api.joint__getTransformFromParentBodyNode

def joint__getTransformFromChildBodyNode(*args) -> "double [4][4]" :
  return _pydart2_api.joint__getTransformFromChildBodyNode(*args)
joint__getTransformFromChildBodyNode = _pydart2_api.joint__getTransformFromChildBodyNode

def joint__setPositionLimitEnforced(*args) -> "void" :
  return _pydart2_api.joint__setPositionLimitEnforced(*args)
joint__setPositionLimitEnforced = _pydart2_api.joint__setPositionLimitEnforced

def joint__isPositionLimitEnforced(*args) -> "bool" :
  return _pydart2_api.joint__isPositionLimitEnforced(*args)
joint__isPositionLimitEnforced = _pydart2_api.joint__isPositionLimitEnforced

def joint__hasPositionLimit(*args) -> "bool" :
  return _pydart2_api.joint__hasPositionLimit(*args)
joint__hasPositionLimit = _pydart2_api.joint__hasPositionLimit

def joint__getPositionLowerLimit(*args) -> "double" :
  return _pydart2_api.joint__getPositionLowerLimit(*args)
joint__getPositionLowerLimit = _pydart2_api.joint__getPositionLowerLimit

def joint__setPositionLowerLimit(*args) -> "void" :
  return _pydart2_api.joint__setPositionLowerLimit(*args)
joint__setPositionLowerLimit = _pydart2_api.joint__setPositionLowerLimit

def joint__getPositionUpperLimit(*args) -> "double" :
  return _pydart2_api.joint__getPositionUpperLimit(*args)
joint__getPositionUpperLimit = _pydart2_api.joint__getPositionUpperLimit

def joint__setPositionUpperLimit(*args) -> "void" :
  return _pydart2_api.joint__setPositionUpperLimit(*args)
joint__setPositionUpperLimit = _pydart2_api.joint__setPositionUpperLimit

def joint__getDof(*args) -> "int" :
  return _pydart2_api.joint__getDof(*args)
joint__getDof = _pydart2_api.joint__getDof

def joint__getNumDofs(*args) -> "int" :
  return _pydart2_api.joint__getNumDofs(*args)
joint__getNumDofs = _pydart2_api.joint__getNumDofs

def joint__getSpringStiffness(*args) -> "double" :
  return _pydart2_api.joint__getSpringStiffness(*args)
joint__getSpringStiffness = _pydart2_api.joint__getSpringStiffness

def joint__setSpringStiffness(*args) -> "void" :
  return _pydart2_api.joint__setSpringStiffness(*args)
joint__setSpringStiffness = _pydart2_api.joint__setSpringStiffness

def joint__getRestPosition(*args) -> "double" :
  return _pydart2_api.joint__getRestPosition(*args)
joint__getRestPosition = _pydart2_api.joint__getRestPosition

def joint__setRestPosition(*args) -> "void" :
  return _pydart2_api.joint__setRestPosition(*args)
joint__setRestPosition = _pydart2_api.joint__setRestPosition

def joint__getDampingCoefficient(*args) -> "double" :
  return _pydart2_api.joint__getDampingCoefficient(*args)
joint__getDampingCoefficient = _pydart2_api.joint__getDampingCoefficient

def joint__setDampingCoefficient(*args) -> "void" :
  return _pydart2_api.joint__setDampingCoefficient(*args)
joint__setDampingCoefficient = _pydart2_api.joint__setDampingCoefficient

def joint__getCoulombFriction(*args) -> "double" :
  return _pydart2_api.joint__getCoulombFriction(*args)
joint__getCoulombFriction = _pydart2_api.joint__getCoulombFriction

def joint__setCoulombFriction(*args) -> "void" :
  return _pydart2_api.joint__setCoulombFriction(*args)
joint__setCoulombFriction = _pydart2_api.joint__setCoulombFriction

def revolute_joint__getAxis(*args) -> "double [3]" :
  return _pydart2_api.revolute_joint__getAxis(*args)
revolute_joint__getAxis = _pydart2_api.revolute_joint__getAxis

def revolute_joint__setAxis(*args) -> "void" :
  return _pydart2_api.revolute_joint__setAxis(*args)
revolute_joint__setAxis = _pydart2_api.revolute_joint__setAxis

def prismatic_joint__getAxis(*args) -> "double [3]" :
  return _pydart2_api.prismatic_joint__getAxis(*args)
prismatic_joint__getAxis = _pydart2_api.prismatic_joint__getAxis

def prismatic_joint__setAxis(*args) -> "void" :
  return _pydart2_api.prismatic_joint__setAxis(*args)
prismatic_joint__setAxis = _pydart2_api.prismatic_joint__setAxis

def universal_joint__getAxis1(*args) -> "double [3]" :
  return _pydart2_api.universal_joint__getAxis1(*args)
universal_joint__getAxis1 = _pydart2_api.universal_joint__getAxis1

def universal_joint__setAxis1(*args) -> "void" :
  return _pydart2_api.universal_joint__setAxis1(*args)
universal_joint__setAxis1 = _pydart2_api.universal_joint__setAxis1

def universal_joint__getAxis2(*args) -> "double [3]" :
  return _pydart2_api.universal_joint__getAxis2(*args)
universal_joint__getAxis2 = _pydart2_api.universal_joint__getAxis2

def universal_joint__setAxis2(*args) -> "void" :
  return _pydart2_api.universal_joint__setAxis2(*args)
universal_joint__setAxis2 = _pydart2_api.universal_joint__setAxis2

def euler_joint__getAxisOrder(*args) -> "char const *" :
  return _pydart2_api.euler_joint__getAxisOrder(*args)
euler_joint__getAxisOrder = _pydart2_api.euler_joint__getAxisOrder

def euler_joint__setAxisOrder(*args) -> "void" :
  return _pydart2_api.euler_joint__setAxisOrder(*args)
euler_joint__setAxisOrder = _pydart2_api.euler_joint__setAxisOrder

def shapenode__getOffset(*args) -> "double [3]" :
  return _pydart2_api.shapenode__getOffset(*args)
shapenode__getOffset = _pydart2_api.shapenode__getOffset

def shapenode__setOffset(*args) -> "void" :
  return _pydart2_api.shapenode__setOffset(*args)
shapenode__setOffset = _pydart2_api.shapenode__setOffset

def shapenode__getRelativeTransform(*args) -> "double [4][4]" :
  return _pydart2_api.shapenode__getRelativeTransform(*args)
shapenode__getRelativeTransform = _pydart2_api.shapenode__getRelativeTransform

def shapenode__setRelativeTransform(*args) -> "void" :
  return _pydart2_api.shapenode__setRelativeTransform(*args)
shapenode__setRelativeTransform = _pydart2_api.shapenode__setRelativeTransform

def shapenode__hasVisualAspect(*args) -> "bool" :
  return _pydart2_api.shapenode__hasVisualAspect(*args)
shapenode__hasVisualAspect = _pydart2_api.shapenode__hasVisualAspect

def shapenode__hasCollisionAspect(*args) -> "bool" :
  return _pydart2_api.shapenode__hasCollisionAspect(*args)
shapenode__hasCollisionAspect = _pydart2_api.shapenode__hasCollisionAspect

def shapenode__getVisualAspectRGBA(*args) -> "double [4]" :
  return _pydart2_api.shapenode__getVisualAspectRGBA(*args)
shapenode__getVisualAspectRGBA = _pydart2_api.shapenode__getVisualAspectRGBA

def shapenode__setVisualAspectRGBA(*args) -> "void" :
  return _pydart2_api.shapenode__setVisualAspectRGBA(*args)
shapenode__setVisualAspectRGBA = _pydart2_api.shapenode__setVisualAspectRGBA

def shape__getVolume(*args) -> "double" :
  return _pydart2_api.shape__getVolume(*args)
shape__getVolume = _pydart2_api.shape__getVolume

def shape__getShapeType(*args) -> "int" :
  return _pydart2_api.shape__getShapeType(*args)
shape__getShapeType = _pydart2_api.shape__getShapeType

def shape__render(*args) -> "void" :
  return _pydart2_api.shape__render(*args)
shape__render = _pydart2_api.shape__render

def shape__getBoundingBoxMin(*args) -> "double [3]" :
  return _pydart2_api.shape__getBoundingBoxMin(*args)
shape__getBoundingBoxMin = _pydart2_api.shape__getBoundingBoxMin

def shape__getBoundingBoxMax(*args) -> "double [3]" :
  return _pydart2_api.shape__getBoundingBoxMax(*args)
shape__getBoundingBoxMax = _pydart2_api.shape__getBoundingBoxMax
# This file is compatible with both classic and new-style classes.





import numpy as np


class SkelVector(np.ndarray):
    def __new__(cls, data=None, skel=None):
        if data is None:
            data = np.zeros(skel.ndofs)
        obj = np.asarray(data).view(cls)
        obj.skel = skel
        obj.keys = [d.name for d in skel.dofs]
        return obj

    def to_index(self, key):
        if isinstance(key, int):
            if key < 0:
                return len(self) - key
            else:
                return key
        elif isinstance(key, str):
            return self.keys.index(key)
        else:
            return None

    def __getitem__(self, key):
        if isinstance(key, str):
            index = self.to_index(key)
            return self.__getitem__(index)
        elif isinstance(key, list) or isinstance(key, tuple):
            indices = [self.to_index(x) for x in key]
            ret = self.__getitem__(np.array(indices))
            ret.keys = [self.keys[i] for i in indices]
            return ret
        else:
            return np.ndarray.__getitem__(self, key)

    def __getslice__(self, start, end):
        ret = np.ndarray.__getslice__(self, start, end)
        ret.keys = self.keys[start:end]
        return ret

    def __setitem__(self, key, value):
        if isinstance(key, str):
            index = self.to_index(key)
            return self.__setitem__(index, value)
        elif isinstance(key, list) or isinstance(key, tuple):
            indices = [self.to_index(x) for x in key]
            return self.__setitem__(np.array(indices), value)
        else:
            return np.ndarray.__setitem__(self, key, value)

    def __contains__(self, key):
        return key in self.keys

    def __array_finalize__(self, obj):
        if obj is None:
            return
        self.skel = getattr(obj, 'skel', None)



import sys


def init_numpy_handler():
    print("register jsonpickle.ext.numpy handlers...")
    try:
        import jsonpickle.ext.numpy
        jsonpickle.ext.numpy.register_handlers()
    except Exception:
        e = sys.exc_info()[1]
        print("exception = " + str(e))
        print("register jsonpickle.ext.numpy handlers... NG")
        return
    print("register jsonpickle.ext.numpy handlers... OK")



from __future__ import print_function
from __future__ import absolute_import
from . import pydart2_api as papi
from . import world
from . import constraints
import os.path
import sys
try:
    from . import gui
except Exception:
    e = sys.exc_info()[1]
    print("-" * 40)
    print("Error while importing pydart2.gui")
    print(e)
    print("-" * 40)

from . import utils
from .world import World
from .skel_vector import SkelVector


__version__ = (0, 5, 0)


def init(verbose=True):
    papi.init(verbose)



from __future__ import absolute_import
# Copyright (c) 2015, Disney Research
# All rights reserved.
#
# Author(s): Sehoon Ha <sehoon.ha@disneyresearch.com>
# Disney Research Robotics Group


from . import viewer
import pydart2.gui.side_panel



from __future__ import absolute_import
from builtins import object
# Copyright (c) 2015, Disney Research
# All rights reserved.
#
# Author(s): Sehoon Ha <sehoon.ha@disneyresearch.com>
# Disney Research Robotics Group
from . import pydart2_api as papi
from .shape import Shape


class ShapeNode(object):
    """
    """

    def __init__(self, _bodynode, _id):
        """
        """
        self.bodynode = _bodynode
        self.skeleton = _bodynode.skeleton
        self.id = _id
        self.shape = Shape(self)

    @property
    def skel(self):
        return self.skeleton

    @property
    def wid(self):
        return self.skel.world.id

    @property
    def skid(self):
        return self.skel.id

    @property
    def bid(self):
        return self.bodynode.id

    def build(self):
        pass

    def has_visual_aspect(self, ):
        return papi.shapenode__hasVisualAspect(self.wid,
                                               self.skid,
                                               self.bid,
                                               self.id)

    def has_collision_aspect(self, ):
        return papi.shapenode__hasCollisionAspect(self.wid,
                                                  self.skid,
                                                  self.bid,
                                                  self.id)

    def visual_aspect_rgba(self, ):
        if not self.has_visual_aspect():
            return None
        return papi.shapenode__getVisualAspectRGBA(self.wid,
                                                   self.skid,
                                                   self.bid,
                                                   self.id)

    def set_visual_aspect_rgba(self, rgba):
        if not self.has_visual_aspect():
            return
        papi.shapenode__setVisualAspectRGBA(self.wid,
                                            self.skid,
                                            self.bid,
                                            self.id,
                                            rgba)

    def offset(self, ):
        return papi.shapenode__getOffset(self.wid,
                                         self.skid,
                                         self.bid,
                                         self.id)

    def set_offset(self, offset):
        papi.shapenode__setOffset(self.wid,
                                  self.skid,
                                  self.bid,
                                  self.id,
                                  offset)

    def relative_transform(self, ):
        return papi.shapenode__getRelativeTransform(self.wid,
                                                    self.skid,
                                                    self.bid,
                                                    self.id)

    def set_relative_transform(self, transform):
        papi.shapenode__setRelativeTransform(self.wid,
                                             self.skid,
                                             self.bid,
                                             self.id, transform)

    def __repr__(self):
        return '[ShapeNode(%d:%d)]' % (self.bid, self.id)
